import { Construct } from 'constructs';
import * as cloud9 from 'aws-cdk-lib/aws-cloud9';
import * as ssm from 'aws-cdk-lib/aws-ssm';
import * as iam from 'aws-cdk-lib/aws-iam';
export interface CustomCloud9SsmProps {
    /**
     * Optional configuration for the SSM Document.
     *
     * @default: none
     */
    readonly ssmDocumentProps?: ssm.CfnDocumentProps;
    /**
     * Optional configuration for the Cloud9 EC2 environment.
     *
     * @default: none
     */
    readonly cloud9Ec2Props?: cloud9.CfnEnvironmentEC2Props;
}
export declare class CustomCloud9Ssm extends Construct {
    private static readonly DEFAULT_EBS_SIZE;
    private static readonly DEFAULT_DOCUMENT_FILE_NAME;
    private static readonly RESIZE_STEP_FILE_NAME;
    private static readonly DEPLOY_CDK_STEP_FILE_NAME;
    private static readonly ATTACH_PROFILE_FILE_NAME;
    private static readonly DEFAULT_DOCUMENT_NAME;
    private readonly document;
    /**
     * The IAM Role that is attached to the EC2 instance launched with the Cloud9 environment to grant it permissions to execute the statements in the SSM Document.
     */
    readonly ec2Role: iam.Role;
    /**
     * Adds one or more steps to the content of the SSM Document.
     * @param steps YAML formatted string containing one or more steps to be added to the mainSteps section of the SSM Document.
     */
    addDocumentSteps(steps: string): void;
    /**
     * Adds one or more parameters to the content of the SSM Document.
     * @param parameters YAML formatted string containing one or more parameters to be added to the parameters section of the SSM Document.
     */
    addDocumentParameters(parameters: string): void;
    /**
     * Adds a step to the SSM Document content that resizes the EBS volume of the EC2 instance. Attaches the required policies to ec2Role.
     * @param size in GiB to resize the EBS volume to.
     */
    resizeEBSTo(size: number): void;
    /**
     * Adds a step to the SSM Document content that deploys a CDK project from its compressed version.
     * @param url from where to download the file using the wget command. Attaches the required policies to ec2Role.
     * @param stackName name of the stack to deploy
     */
    deployCDKProject(url: string, stackName?: string): void;
    constructor(scope: Construct, id: string, props?: CustomCloud9SsmProps);
}
